/***************************************************************************
 *   Copyright (C) 2004-2005 by Jürgen Kofler                                   *
 *   kaffeine@gmx.net                                                      *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin Steet, Fifth Floor, Boston, MA 02110-1301, USA.           *
 ***************************************************************************/

/*
 * Last modified: $Date: 2005/09/03 09:36:40 $ by $Author: juergenk $
 */

#ifndef _KAFFEINE_H_
#define _KAFFEINE_H_

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <kapplication.h>
#include <kfiledialog.h>
#include <kmainwindow.h>
#include <kparts/part.h>

#include <qptrlist.h>
#include <qtimer.h>
#include <qsplitter.h>

#include "kaffeineiface.h"

class KToggleAction;
class KActionMenu;
class KRecentFilesAction;
class KToggleFullScreenAction;
class KMediaPart;
class PlayList;
class StartWindow;
class SystemTray;
class MRL;
class TabWidget;
class PlayerContainer;
#ifdef HAVE_LINUX_DVB_FRONTEND_H
class DvbPanel;
#endif
#include "dvbclient/cdwidget.h"

/**
 *
 * @author Jürgen Kofler <kaffeine@gmx.net>
 *
 */
class Kaffeine : public virtual KMainWindow, public virtual KaffeineIface
{
	Q_OBJECT
public:
	Kaffeine(const QStringList& urls = QStringList(), const QStringList& engineParameters = QStringList(),
	         const QString device = QString::null, bool instantPlay = false, bool fullscreen = false, bool wizard = false,
	         QWidget* parent = 0, const char* name = 0);
	virtual ~Kaffeine();

	/**
	 * Use this method to load whatever file/URL you have
	 */
	virtual void load(const QStringList&);
	virtual void load(const QString&);
	/**
	  * Load urls from command line into the "NEW" playlist
	  * @returns if any url like "DVD", "VCD" was found
	  */
	virtual bool loadTMP(const QStringList&);

	virtual void setDevice(const QString& device) { m_device = device; }

	/*
	 * DCOP functions...
	 */
	virtual void openURL(QString url);
	virtual void appendURL(QString url);
	virtual void play();
	virtual void playAudioCD();
	virtual void playVCD();
	virtual void playDVD();
	virtual void pause();
	virtual void stop();
	virtual void next();
	virtual void previous();
	virtual void changePlaylist();
	virtual bool isPlaying();
	virtual bool isVideo();
	virtual QString title();
	virtual QString artist();
	virtual QString album();
	virtual QString track();
	virtual QString getFileName();
	virtual void random();
	virtual void fullscreen();
	virtual int getLength();
	virtual int getTimePos();
	virtual void posPlus();
	virtual void posMinus();
	virtual void volUp();
	virtual void volDown();
	virtual void mute();
	virtual void quit();
	virtual void setNumber( int num );
	virtual void dvbOSD();

public slots:
	virtual void slotPlay(const MRL&);
	virtual void slotLoadURLS(const QStringList&);
	virtual void slotSwitchToPlayerWindow();
	virtual void slotSetActiveBrowser( const QString& );

protected:
	virtual void showEvent(QShowEvent*);
	virtual void hideEvent(QHideEvent*);

protected:
	virtual void closeEvent(QCloseEvent*);
	//  void dragEnterEvent(QDragEnterEvent*);
	//  void dropEvent(QDropEvent*);
	virtual void mouseDoubleClickEvent(QMouseEvent*);
	virtual void mouseMoveEvent(QMouseEvent*);
	virtual void mousePressEvent(QMouseEvent*);
	virtual void resizeEvent(QResizeEvent*);

private slots:
	virtual void slotOpenFile();
	virtual void slotQuit();
	virtual void optionsConfigureKeys();
	virtual void optionsConfigureToolbars();
	virtual void applyNewToolbarConfig();
	virtual void slotFakeKeyEvent();
	virtual void slotSetScreensaverTimeout(int);
	virtual void slotChangeCaption(const QString&);
	virtual void slotChangeStatusbar(const QString&);
	virtual void slotChangePlaylistStatus(const QString&);
	virtual void slotOpenURL();
	virtual void slotOpenDirectory();
	virtual void slotOpenDVD();
	virtual void slotOpenVCD();
	virtual void slotOpenAudioCD();
	virtual void slotPlaylistPlay();
	virtual void slotPlaylistNext();
	virtual void slotPlaylistPrevious();
	virtual void slotPlaylistNew();
	virtual void slotPlaylistLoad();
	virtual void slotPlaylistSaveAs();
	virtual void slotPlaylistRemove();
	virtual void slotRequestForCurrentTrack();
	virtual void slotRequestForNextTrack();
	virtual void slotRequestForPreviousTrack();
	virtual void slotRequestForTrackNumber();
	virtual void slotRepeat();
	virtual void slotShuffle();
	virtual void slotPlayRecent(const KURL&);
	virtual void slotMetaFromPlayer();
	virtual void slotToggleFullscreen();
	virtual void slotEscapeFullscreen();
	virtual void slotToggleMinimalMode();
	virtual void slotToggleTab();
	virtual void slotPreferences();
	virtual void slotOriginalAspect();
	virtual void slotAutoresizeOff();
	virtual void slotAutoresizeOriginal();
	virtual void slotAutoresizeDouble();
	virtual void slotAutoresizeTriple();
	virtual void slotNewFrameSize(const QSize&);
	virtual void slotCurrentTabChanged(QWidget*);
	virtual void slotClearRecent();
	virtual void slotLoadPart(const QString&);
	virtual void slotLoadingCanceled(const QString&);
	virtual void slotPlaybackFailed();
	virtual void slotSystemTray(bool);
	virtual void slotUseAlternateEncoding(bool);
	virtual void slotAlternateEncoding(const QString&);
	virtual void slotStop();
	virtual void slotMute();
	virtual void slotSleepAfterPlay();
	virtual void slotSleepAfterPlayMenu();
	virtual void slotQuitAfterPlay();
	virtual void slotQuitAfterPlayMenu();
	virtual void slotQuitAfterPlaylistMenu();
	virtual void slotTogglePlaylistDockState();
	virtual void slotShowGoTab(bool);
	virtual void slotSetOSDTimeout(uint);
	virtual void slotPauseVideo(bool);
	virtual void slotDvbClient(bool,const QString&,int,int,const QString&);
	virtual void slotNumKeyInput( int );

private:
	virtual void autoresize();
	virtual void setupAccel();
	virtual void setupActions();
	virtual void loadConfig();
	virtual void saveConfig();
	virtual void hideToolbars(bool);
	virtual QString askForOtherDevice(const QString&);
	virtual void unloadCurrentPart();

private:
	KMediaPart* m_mediaPart;
	KParts::ReadOnlyPart* m_simplePart;
	PlayerContainer* m_playerContainer;
	PlayList* m_playlist;
	StartWindow* m_startWindow;
#ifdef HAVE_LINUX_DVB_FRONTEND_H

	DvbPanel *dvbPanel;
#endif

	CdWidget *dvbClient;
	bool m_dvbClientEnabled;
	QString m_dvbClientAddress;
	int m_dvbClientPort;
	int m_dvbClientInfo;
	QString m_dvbClientShiftPath;
	TabWidget* m_tabWidget;
	SystemTray* m_systemTray;
	QSize m_videoSize;
	bool m_noResize;
	bool m_embedSystemTray;
	bool m_sleepInfo;
	uint m_osdDuration;
	uint m_autoResizeFactor;
	QString m_activeBrowser;

	bool m_useAlternateEncoding;
	QString m_alternateEncoding;

	bool m_autoPaused;
	bool m_pauseVideo;

	bool m_haveKWin;

	QString m_currentPartService;

	QString m_device;
	QString m_filter;
	QStringList m_engineParameters;

	KRecentFilesAction* m_recent;
	KToggleFullScreenAction* m_fullscreen;
	KToggleAction* m_minimal;
	KToggleAction* m_shuffle;
	KToggleAction* m_repeat;
	KToggleAction* m_autoResizeOff;
	KToggleAction* m_autoResizeOriginal;
	KToggleAction* m_autoResizeDouble;
	KToggleAction* m_autoResizeTriple;
	KToggleAction* m_originalAspect;
	KActionMenu* m_playersMenu;
	KToggleAction* m_sleepAfterPlay;
	KToggleAction* m_quitAfterPlay;
	KToggleAction* m_quitAfterPlaylist;
	bool m_statusBarVisible;

	QTimer m_screensaverTimer;
	int m_screensaverTimeout;
	QTimer m_numKeyHideTimer;
	int m_numKey;
	QString m_captionCache;
	int m_haveXTest, m_xTestKeycode;

signals:
	virtual void showOSD( const QString &text, int duration, int priority );
};

#include <ktabwidget.h>

class TabWidget : public KTabWidget
{

public:
	TabWidget(QWidget* parent = 0, const char* name = 0);
	virtual ~TabWidget()
	{}

	void showTabBar();
	void hideTabBar();

};

#include <qvbox.h>

class PlayerContainer : public QVBox
{
	Q_OBJECT

public:
	PlayerContainer(QWidget* parent = 0, const char* name = 0);
	virtual ~PlayerContainer()
	{}

signals:
	void signalURLDropEvent(const QStringList&);

protected:
	void dragEnterEvent (QDragEnterEvent*);
	void dropEvent(QDropEvent*);

};

#endif // _KAFFEINE_H_
